% release version of togAggregateResults
% 
% Aggregates results generated with towersOfGold_Release.m

%%
clear, clc

%% setup parameters

dataPath = '~/Downloads/ToG_Data/Cells';

% viewDirection = [1 0 0]; % x,y,z
viewDirection = [120 30]; % azimuth, elevation (top: [90 90], side: [90 0])
backgroundColor = 'black';
gpMarker = '*';
gpMarkerSize = 1;
nAzBins = 4; % n. azimuth bins
nHtBins = 5; % n. height bins; last bin corresponds to gold particles of z position > tower height
towersAsPointClouds = false; % if false, show convex hull's triangulated mesh
trMarkerSize = 1; % only relevant when towersAsPointClouds = true

%% inspect folder

listing = dir(dataPath);
nFolders = 0;
for i = 1:size(listing,1)
    if listing(i).isdir && ~strcmp(listing(i).name(1),'.')
        nFolders = nFolders+1;
        folderNames{nFolders} = listing(i).name;
    end
end

%% read data

allAT = cell(1,4);
allAGP = cell(1,4);
allSC = zeros(4,2,nFolders);

for iFolder = 1:nFolders
    T = readtable([dataPath filesep folderNames{iFolder} filesep 'Tables' filesep 'AvgHeightAndRadius.xls']);
    A = table2array(T);
    allSC(:,:,iFolder) = A(:,2:3);
    for row = 1:4
        if exist([dataPath filesep folderNames{iFolder} filesep 'Tables' filesep sprintf('AgrTwrRow%d.csv',row)],'file')
            TableAT = readtable([dataPath filesep folderNames{iFolder} filesep 'Tables' filesep sprintf('AgrTwrRow%d.csv',row)]);
            AT = table2array(TableAT);
            AT(:,3) = AT(:,3)/A(row,2);
            AT(:,1:2) = AT(:,1:2)/A(row,3);
            aat = allAT{row};
            aat = [aat; AT];
            allAT{row} = aat;
        end
        if exist([dataPath filesep folderNames{iFolder} filesep 'Tables' filesep sprintf('AgrGPRow%d.csv',row)],'file')
            TableAGP = readtable([dataPath filesep folderNames{iFolder} filesep 'Tables' filesep sprintf('AgrGPRow%d.csv',row)]);
            AGP = table2array(TableAGP);
            AGP(:,3) = AGP(:,3)/A(row,2);
            AGP(:,1:2) = AGP(:,1:2)/A(row,3);
            aagp = allAGP{row};
            aagp = [aagp; AGP];
            allAGP{row} = aagp;
        end
    end
end

ts = [];
for row = 1:4
    meanHeight = mean(allSC(row,1,:),'omitnan');
    meanRad = mean(allSC(row,2,:),'omitnan');
    
    allATrow = allAT{row};
    if ~isempty(allATrow)
        allATrow(:,3) = allATrow(:,3)*meanHeight;
        allATrow(:,1:2) = allATrow(:,1:2)*meanRad;
        allAT{row} = allATrow;
        ts = [ts; allATrow];
    end
    
    allAGProw = allAGP{row};
    if ~isempty(allAGProw)
        allAGProw(:,3) = allAGProw(:,3)*meanHeight;
        allAGProw(:,1:2) = allAGProw(:,1:2)*meanRad;
        allAGP{row} = allAGProw;
        ts = [ts; allAGProw];
    end
end


%% display aggregated data

figureQSS
axs = [];
for row = 1:4
    ax = subplot(1,4,row); hold on
    axs = [axs ax];
end
mnts = min(ts);
mxts = max(ts);
mmts = max([abs(mnts); abs(mxts)]);
mmtsxy = max(mmts(1:2));
x = 0.9*mmtsxy*cos(linspace(0,2*pi,36));
y = 0.9*mmtsxy*sin(linspace(0,2*pi,36));
z = zeros(size(x));

for row = 1:4
    subplot(1,4,row)
    plot3(x,y,z,'-b','LineWidth',2)
    plot3([0 0],[0 0.9*mmtsxy],[0 0],'-b','LineWidth',2)
    AT = allAT{row};
    if ~isempty(AT)
        if towersAsPointClouds
            plot3(AT(:,1),AT(:,2),AT(:,3),'.','Color',[1 1 1],'MarkerSize',trMarkerSize);
        else
            k = boundary(AT(:,1),AT(:,2),AT(:,3),0);
            trisurf(k,AT(:,1),AT(:,2),AT(:,3), 'Facecolor','white','FaceAlpha',0.25,'EdgeColor','white','EdgeAlpha',0.25);
        end
    end
    AGP = allAGP{row};
    if ~isempty(AGP)
        plot3(AGP(:,1),AGP(:,2),AGP(:,3),gpMarker,'Color',[1 1 0],'MarkerSize',gpMarkerSize);
    end
    hold off
    axis equal
    axis([-mmtsxy mmtsxy -mmtsxy mmtsxy 0 mmts(3)])
    view(viewDirection)
    xlabel('x (um)'), ylabel('y (um)'), zlabel('z (um)')
    title(sprintf('row %d\navg ht: %.1f\navg rd: %.1f',row,mean(allSC(row,1,:),'omitnan'),mean(allSC(row,2,:),'omitnan')));
    set(gca,'Color',backgroundColor)
end

Link = linkprop(axs,{'CameraUpVector','CameraPosition','CameraTarget'});
setappdata(gcf,'StoreTheLink',Link)
rotate3d on
saveas(gcf,[dataPath filesep 'AgrTowersAndGP.fig']);

%% compute aggregated cylinder histogram

angleInterval = 2*pi/nAzBins;
angles = pi/2+(0:angleInterval:2*pi-angleInterval);
vectors = [cos(angles); sin(angles)];

H = zeros(nHtBins,nAzBins,4);

for row = 1:4
    AGP = allAGP{row};
    if ~isempty(AGP)
        avgHt = mean(allSC(row,1,:),'omitnan');
        heightInterval = avgHt/(nHtBins-1);
        xy = AGP(:,1:2);
        dp = zeros(size(xy,1),nAzBins);
        for iAzBin = 1:nAzBins
            dp(:,iAzBin) = sum(xy.*repmat(vectors(:,iAzBin)',[size(xy,1) 1]),2);
        end
        [~,aBin] = max(dp,[],2);
        hBin = floor(AGP(:,3)/heightInterval)+1;
        hBin(hBin >= nHtBins) = nHtBins;
        hBin(hBin == 0) = 1;
        for iGP = 1:size(AGP,1)
            i = nHtBins+1-hBin(iGP);
            j = aBin(iGP);
            H(i,j,row) = H(i,j,row)+1;
        end
    end
%     [size(AGP,1) sum(sum(H(:,:,row)))]
end

% m = max(H(:));
figureQSS
for row = 1:4
    h = H(:,:,row);
    summaryCell = cell(nHtBins,nAzBins+1);
    for i = 1:nHtBins
        summaryCell{i,1} = sprintf('ht_bin_%d',nHtBins-i+1);
        for j = 1:nAzBins
            summaryCell{i,1+j} = h(i,j);
        end
    end
    variableNames = cell(1,nAzBins);
    variableNames{1} = 'hist';
    for j = 1:nAzBins
        variableNames{j+1} = sprintf('az_bin_%d', j);
    end
    T = cell2table(summaryCell,'VariableNames',variableNames);
    writetable(T,[dataPath filesep sprintf('CylinderHistogramRow%d.xls',row)]);
    
    m = max(H(:));
    h = h/m;
    subplot(1,4,row)
    I = imresize(h,100,'nearest');
    Hue = 2/3*(1-I);
    S = ones(size(I));
    V = ones(size(I));
    HSV = cat(3,Hue,cat(3,S,V));
    RGB = hsv2rgb(HSV);
    imshow(RGB), title(sprintf('row: %d',row))
    if row == 1
        xlabel('az sector, ccw from [0 1 0]')
        ylabel('ht sector, up-most: ht > tr ht')
    end
end

saveas(gcf,[dataPath filesep 'CylinderHistograms.fig']);

%% display polar histogram

gr = -1:0.0025:1;
[X,Y] = meshgrid(gr,gr);
Y = flipud(Y);
D = sqrt(X.^2+Y.^2);

xy = [X(:) Y(:)];
dp = zeros(size(xy,1),nAzBins);
for iAzBin = 1:nAzBins
    dp(:,iAzBin) = sum(xy.*repmat(vectors(:,iAzBin)',[size(xy,1) 1]),2);
end
[~,aBin] = max(dp,[],2);

Z = reshape(aBin,size(X));
S = ones(size(X));
V = ones(size(X));

radii = (1/(nHtBins+1):1/(nHtBins+1):1)*size(X,1)/2;
cs = zeros(length(radii),3);
for i = 1:length(radii)
    cs(i,:) = [size(X,1)/2 size(X,1)/2 radii(i)];
end
ls = cell(1,nAzBins);
bAngles = angles-pi/nAzBins;
bVectors = [cos(bAngles); sin(bAngles)];
for i = 1:nAzBins
    x1 = size(X,1)/2+radii(1)*bVectors(1,i);
    y1 = size(X,1)/2-radii(1)*bVectors(2,i);
    x2 = size(X,1)/2+radii(end)*bVectors(1,i);
    y2 = size(X,1)/2-radii(end)*bVectors(2,i);
    ls{i} = [x1 y1 x2 y2];
end

figureQSS
for row = 1:4
    I = zeros(size(X));
    for hBin = 1:nHtBins    
        rg = [hBin/(nHtBins+1) (hBin+1)/(nHtBins+1)];
        for i = 1:size(I,1)
            for j = 1:size(I,2)
                if D(i,j) >= rg(1) && D(i,j) < rg(2)
                    I(i,j) = H(hBin,Z(i,j),row);
                end
            end
        end
    end
    I = 2/3*(1-I/max(H(:)));
    Hue = I;
    HSV = cat(3,Hue,cat(3,S,V));
    RGB = hsv2rgb(HSV);
    for i = 1:size(I,1)
        for j = 1:size(I,2)
            if D(i,j) < 1/(nHtBins+1) || D(i,j) >= 1
                RGB(i,j,:) = 0;
            end
        end
    end
    
    RGB = insertShape(RGB,'Line',ls,'LineWidth',6,'Color','black');
    RGB = insertShape(RGB,'circle',cs,'LineWidth',6,'Color','black');
    
    HueCM = repmat(linspace(0,2/3,size(RGB,1))',[1 16]);
    SVCM = ones(size(HueCM,1),size(HueCM,2),2);
    CM = hsv2rgb(cat(3,HueCM,SVCM));
    
    subplot(1,4,row)
    RGBCM = [RGB zeros(size(RGB,1),6,3) CM]; 
    imshow(RGBCM)
    imwrite(RGBCM,[dataPath filesep sprintf('CylinderHistogramRow%d.png',row)]);
    title(sprintf('row: %d',row))
    if row == 1
        xlabel('x'), ylabel('y')
    end
end