% release version of togDisplayResults
% 
% Displays results generaged with towersOfGold_Release.m

%%
clear, clc

%% setup parameters

dataPath = '~/Downloads/ToG_Data/Cells/50687';

% viewDirection = [1 0 0];
viewDirection = [90 0]; % azimuth, elevation (top: [90 90], side: [90 0])
backgroundColor = 'black';
gpMarker = '*';
gpMarkerSize = 1;
towersAsPointClouds = false; % if false, show convex hull's triangulated mesh
trMarkerSize = 1; % only relevant when towersAsPointClouds = true

%%

allAT = cell(1,4);
allAGP = cell(1,4);

ts = [];
for row = 1:4
    if exist([dataPath filesep 'Tables' filesep sprintf('AgrTwrRow%d.csv',row)],'file')
        TableAT = readtable([dataPath filesep 'Tables' filesep sprintf('AgrTwrRow%d.csv',row)]);
        AT = table2array(TableAT);
        allAT{row} = AT;
        ts = [ts; AT];
    end
    if exist([dataPath filesep 'Tables' filesep sprintf('AgrGPRow%d.csv',row)],'file')
        TableAGP = readtable([dataPath filesep 'Tables' filesep sprintf('AgrGPRow%d.csv',row)]);
        AGP = table2array(TableAGP);
        allAGP{row} = AGP;
        ts = [ts; AGP];
    end
end
T = readtable([dataPath filesep 'Tables' filesep 'AvgHeightAndRadius.xls']);
summaryCell = table2cell(T);

%%

figureQSS
axs = [];
for row = 1:4
    ax = subplot(1,4,row); hold on
    axs = [axs ax];
end
mnts = min(ts);
mxts = max(ts);
mmts = max([abs(mnts); abs(mxts)]);
mmtsxy = max(mmts(1:2));
% [x,y,z] = cylinder(0.9*mmtsxy*ones(1,5),12);
% z = mmts(3)*z/5;
x = 0.9*mmtsxy*cos(linspace(0,2*pi,36));
y = 0.9*mmtsxy*sin(linspace(0,2*pi,36));
z = zeros(size(x));

for row = 1:4
    subplot(1,4,row)
%     surf(x,y,z,'EdgeColor','blue'), alpha(0.1)
    plot3(x,y,z,'-b','LineWidth',2)
    plot3([0 0],[0 0.9*mmtsxy],[0 0],'-b','LineWidth',2)
    AT = allAT{row};
    if ~isempty(AT)
        if towersAsPointClouds
            plot3(AT(:,1),AT(:,2),AT(:,3),'.','Color',[1 1 1],'MarkerSize',trMarkerSize);
        else
            k = boundary(AT(:,1),AT(:,2),AT(:,3),0);
            trisurf(k,AT(:,1),AT(:,2),AT(:,3), 'Facecolor','white','FaceAlpha',0.25,'EdgeColor','white','EdgeAlpha',0.25);
        end
    end
    AGP = allAGP{row};
    if ~isempty(AGP)
        plot3(AGP(:,1),AGP(:,2),AGP(:,3),gpMarker,'Color',[1 1 0],'MarkerSize',gpMarkerSize);
    end
    hold off
    axis equal
    axis([-mmtsxy mmtsxy -mmtsxy mmtsxy 0 mmts(3)])
    view(viewDirection)
    xlabel('x (um)'), ylabel('y (um)'), zlabel('z (um)')
    title(sprintf('row %d\navg ht: %.1f\navg rd: %.1f',row,summaryCell{row,2},summaryCell{row,3}));
    set(gca,'Color',backgroundColor)
end

Link = linkprop(axs,{'CameraUpVector','CameraPosition','CameraTarget'});
setappdata(gcf,'StoreTheLink',Link)
rotate3d on